indexing
	description
		"Some examples of the Xalan library features."

class
	MAIN

inherit
	XML

create
	make

feature -- Creation

	make is

		do
			modify_dom
			wrap_dom
			display_utf16
			use_xpath
			transform
		end

feature -- Example code

	doc: DOM_DOCUMENT
	xdoc: XALAN_DOCUMENT
	xpath: XPATH

	modify_dom is
		-- DOM modification is done via a DOM_DOCUMENT.
		local
			root: DOM_ELEMENT
		do
			doc := xml.dom_parse_file ("examples/sample.xml")
			root := doc.document_element
			root.set_attribute ("modified", "true")
			io.put_string (doc.serialize)
		end

	wrap_dom is
		-- A XALAN_DOCUMENT is read only, and can wrap a DOM_DOCUMENT.
		-- If the DOM is modified, then the wrapped XALAN_DOCUMENT
		-- must be recreated.
		-- A XPATH can be used to evaluate an expression
		do
			xdoc := doc.xdoc
			xpath := xdoc.new_xpath ("/sample")
			io.put_string ("%Nmodified = " + xpath.eval ("@modified") + "%N")
		end

	display_utf16 is
		-- UTF16 is the native character format for Xalan & Xerces
		-- If RTL text is stored in collation order then
		-- it must be reversed for console display.
		-- Language & locale should be set appropriately:
		-- export LANG=en_US.UTF-8
		-- export LOCALE=UTF-8
		local
			s: XALAN_DOM_STRING
		do
			s := xpath.eval_utf ("hebrew")
			s.reverse
			io.put_string ("%Nhello:" + s.to_string + "%N")
		end

	use_xpath is
		-- How to select a set of nodes:
		local
			nrl: NODE_REF_LIST
			i: INTEGER
		do
			io.put_string ("%N")
			nrl := xpath.select_nodes ("child")
			from i := 0
			until i >= nrl.count
			loop
				io.put_string (
					nrl.item (i).first_child.node_value + "%N"
				)
				i := i + 1
			end
		end

	transform is

		local
			t: XALAN_TRANSFORMER
		do
			create t.make
			t.set_in_xerces (doc)
			t.set_xslt_file ("examples/sort.xsl")
			t.set_out_string
			t.transform

			io.put_string ("%N")
			io.put_string (t.output)
		end

end
