indexing
	description:
		"Polymorphic list class."
		"Objects in the list must be of a descendant type"
		"of the generic class."

class
	PLIST[G]

create
	make

feature -- Creation

	make is

		do
		end

feature -- Access

	first, last: PNODE[G]
	count: INTEGER

	new_cursor: PCURSOR[G] is
			-- Create a cursor that can traverse the list.
		do
			create Result.make (Current)
		end

	put_first (object: G) is
			-- Insert the object at the start of the list.
		local
			node: PNODE[G]
		do
			create node.make (object)
			if last = Void then
				last := node
			else
				node.set_next (first)
			end
			first := node
			count := count + 1
		end

	put_last (object: G) is
			-- Add the object to the end of the list.
		local
			node: PNODE[G]
		do
			create node.make (object)
			if last = Void then
				first := node
			else
				last.set_next (node)
			end
			last := node
			count := count + 1
		end

	append_last (other: PLIST[G]) is
			-- Append the other list to the end of this list.
		require
			other /= Void
		do
			if other.count > 0 then
				if last = Void then
					first := other.first
				else
					last.set_next (other.first)
				end
				last := other.last
				count := count + other.count
			end
		end

	remove_before (cursor: PCURSOR[G]) is
			-- Remove all nodes before the cursor
		require
			simple: not cursor.after
		local
			c: PCURSOR[G]
		do
			first := cursor.node
			count := 0
			from c := cursor.twin
			until c.after
			loop
				count := count + 1
				c.forth
			end
		end

	remove (index: INTEGER) is
			-- Remove the object at the given position.
		require
			not_empty: count > 0
			valid_index: index >= 0 and index < count
		local
			node: PNODE[G]
		do
			count := count - 1
			if count = 0 then
				first := Void
				last := Void
			else
				if index = 0 then
					first := first.next
				else
					node := first.nth (index - 1)
					if index = count then
						last := node
					end
					node.set_next (node.next.next)
				end
			end
		end

	remove_last is
			-- Remove the last object from the list.
		require
			not_empty: count > 0
		do
			remove (count - 1)
		end

	put, replace (object: G; index: INTEGER) is
			-- Replace the object at the given position.
		require
			in_range: index >= 0 and index < count
		local
			node: PNODE[G]
		do
			node := first.nth (index)
			node.set_item (object)
		end

	infix "@" (index: INTEGER): G is
			-- Get the object at the given position.
		require
			in_range: index >= 0 and index < count
			not_empty: first /= Void
		local
			node: PNODE[G]
		do
			node := first.nth (index)
			Result := node.item
		end

feature {PCURSOR}

	remove_node (node: PNODE[G]) is
			--
		require
			not_empty: count > 0
			has_arg: node /= Void
		local
			prev, test: PNODE[G]
		do
			old_count := count
			if first = node then
				first := first.next
				count := count -1
				if count = 0 then
					last := Void
				end
			else
				from
					prev := first
					test := first.next
				until
					test = node or test = Void
				loop
					prev := test
					test := test.next
				end
				if test = node then
					count := count -1
					prev.set_next (test.next)
					if last = test then
						last := prev
					end
				end
			end
		ensure
			removed: count = old_count - 1
		end

feature {} -- Private

	old_count: INTEGER

invariant
	both_ends: (first = Void and last = Void) or (first /= Void and last /= Void)
	count_valid: (count = 0 and first = Void) or (count > 0 and first /= Void)
end
