indexing
	description:
		"Xerces XML document interface"
		"Use xml.dom_parse_file to read from a file"

class
	DOM_DOCUMENT

inherit
	XML
		undefine
			is_equal
		end
	DISPOSABLE
		undefine
			is_equal
		end
	DOM_NODE
		rename
			make as dom_node_make
		end

create
	make, make_releasable, make_reference

feature -- Creation

	make (root_tag, dtd: STRING) is
			-- making with a DTD probably doesn't work.
		local
			empty: STRING
		do
			xml.initialize
			if dtd = Void  then
				if root_tag = Void then
					make_releasable (new_dom_doc (
						default_pointer,
						default_pointer,
						default_pointer
					))
				else
					make_releasable (new_dom_doc (
						terminated (root_tag),
						default_pointer,
						default_pointer
					))
				end
			else
				create empty.make_empty
				make_releasable (new_dom_doc (
					terminated (root_tag),
					terminated (empty),
					terminated (dtd)
				))
			end
		end

	make_releasable (cpp_object: POINTER) is
			--
		do
			dom_node_make (cpp_object)
			release_doc := true
		end

	make_reference (cpp_object: POINTER) is
			-- C++ object will not be released when this object is disposed of.
		do
			dom_node_make (cpp_object)
		end

	dispose is
			--
		do
			dispose_xalan
			if obj /= default_pointer then
				if release_doc then
					release_dom_doc (obj)
				end
				obj := default_pointer
			end
		end

feature -- Access

	ok: BOOLEAN -- set by write

	create_node (object: POINTER): DOM_NODE is

		do
			Result := xml.create_dom_node (object)
		end

	xdoc: XALAN_DOCUMENT is
			--
		do
			if xalan_document = Void then
				create xalan_document.make_from_xerces (Current)
			end
			Result := xalan_document
		end

	transform (xslt: STRING): STRING is
			--
		local
			xt: XALAN_TRANSFORMER
		do
			create xt.make
			xt.set_in_xerces (Current)
			xt.set_xslt_file (xslt)
			xt.set_out_string
			xt.transform
			Result := xt.output
		end

	document_element: DOM_ELEMENT is

		do
			create Result.make (dd_get_document_element (obj))
		end

	create_element (tag_name: STRING): DOM_ELEMENT is

		do
			create Result.make (dd_create_element (obj, terminated (tag_name)))
		end

	create_text_node (utf8: STRING): DOM_TEXT is

		do
			create Result.make (dd_create_text_node (obj, terminated (utf8)))
		end

	create_utf_text_node (utf16: XALAN_DOM_STRING): DOM_TEXT is

		do
			create Result.make (dd_create_utf16_text_node (obj, utf16.obj))
		end

	write (file_name: STRING) is

		do
			xml.use_formatter_with_file (file_name)
			xml.ftx.serialize (Current)
			xml.dispose_formatter
		end

	serialize: STRING is

		do
			xml.use_formatter_with_string
			xml.ftx.serialize (Current)
			xml.dispose_formatter
			Result := xml.ftx_output
		end

	add_element (parent: DOM_ELEMENT; name, value: STRING) is
			--
		local
			e: DOM_ELEMENT
		do
			e := create_element (name)
			parent.append_child (e)
			e.append_child (create_text_node (value))
		end

	add_utf_element (parent: DOM_ELEMENT; name: STRING; value: XALAN_DOM_STRING) is
			--
		local
			e: DOM_ELEMENT
		do
			e := create_element (name)
			parent.append_child (e)
			e.append_child (create_utf_text_node (value))
		end

	import_node (node: DOM_NODE; deep: BOOLEAN): DOM_NODE is

		do
			Result := xml.create_dom_node (
				dd_import_node (obj, node.obj, deep)
			)
		end

	dispose_xalan is
			--
		do
			if xalan_document /= Void then
				xalan_document.dispose
				xalan_document := Void
			end
		end

feature {} -- Private

	xalan_document: XALAN_DOCUMENT
	release_doc: BOOLEAN

feature {} -- externals

	new_dom_doc (qualified_name, public_id, system_id: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	release_dom_doc (doc: POINTER) is

		external
			"C | %"dom_support.h%""
		end

	dd_create_element (object, tag_name: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dd_create_text_node (object, utf8: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dd_create_utf16_text_node (object, utf16: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dd_import_node (object, node: POINTER; deep: BOOLEAN): POINTER is
			-- import DOMNode(s)
		external
			"C | %"dom_support.h%""
		end

	dd_get_document_element (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

end
