indexing
	description:
		"Xerces DOMNode interface"

class
	DOM_NODE

inherit
	XML
		undefine
			is_equal
		end
	NODE_TYPES
		export
			{ANY} rv
		undefine
			get_transcoded
		redefine
			is_equal
		end

create
	make

feature {DOM_NODE} -- Creation

	make (object: POINTER) is

		require
			valid: object /= default_pointer
		do
			obj := object
		end

feature -- Access

	obj: POINTER

	owner_document: DOM_DOCUMENT is
			--
		do
			Result := xml.node_factory.referenced_document (dn_get_owner_document (obj))
		end

	is_equal (other: like Current): BOOLEAN is

		do
			Result := obj = other.obj
		end

	node_name: STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		do
			Result := get_transcoded (dn_get_node_name (obj))
		end

	node_prefix: STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		do
			Result := get_transcoded (dn_get_node_prefix (obj))
		end

	node_value: STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		do
			Result := get_transcoded (dn_get_node_value (obj))
		end

	node_type: INTEGER is

		do
			Result := dn_get_node_type (obj)
		end

	parent_node: DOM_NODE is

		do
			Result := xml.create_dom_node (dn_get_parent_node (obj))
		end

	child_nodes: DOM_NODE_LIST is

		do
			create Result.make (dn_get_child_nodes (obj))
		end

	first_child: DOM_NODE is

		do
			Result := xml.create_dom_node (dn_get_first_child (obj))
		end

	last_child: DOM_NODE is

		do
			Result := xml.create_dom_node (dn_get_last_child (obj))
		end

	previous_sibling: DOM_NODE is

		do
			Result := xml.create_dom_node (dn_get_previous_sibling (obj))
		end

	next_sibling: DOM_NODE is

		do
			Result := xml.create_dom_node (dn_get_next_sibling (obj))
		end

	clone_node (deep: BOOLEAN): DOM_NODE is

		do
			Result := xml.create_dom_node (dn_clone_node (obj, deep))
		end

	insert_before (new_child: DOM_NODE; ref_child: DOM_NODE) is

		local
			r: POINTER
		do
			r := dn_insert_before (obj, new_child.obj, ref_child.obj)
		end

	replace_child (new_child: DOM_NODE; old_child: DOM_NODE) is

		local
			r: POINTER
		do
			r := dn_replace_child (obj, new_child.obj, old_child.obj)
		end

	remove_child (old_child: DOM_NODE) is

		local
			r: POINTER
		do
			r := dn_remove_child (obj, old_child.obj)
		end

	append_child (new_child: DOM_NODE) is

		local
			r: POINTER
		do
			r := dn_append_child (obj, new_child.obj)
		end

	has_child_nodes: BOOLEAN is

		do
			Result := dn_has_child_nodes (obj) /= 0
		end

	set_node_value (t: STRING) is

		do
			dn_set_node_value (obj, terminated (t))
		end

feature -- Experimental

	for_each_child (action: PROCEDURE [ANY, TUPLE [DOM_NODE, DOM_NODE]]; target: DOM_NODE) is
			-- call agent for each child of this node
		local
			child: DOM_NODE
		do
			from child := first_child
			until child = Void
			loop
				action.call ([child, target])
				child := child.next_sibling
			end
		end

feature {} -- Private

	get_transcoded (length: INTEGER): STRING is
			-- extra char for null
		do
			if length >= 0 then
				create Result.make_filled (' ', length - 1)
				copy_transcode_buffer (str_to_ptr (Result), length - 1)
			end
		end

feature {} --Externals

	dn_get_node_name (object: POINTER): INTEGER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_node_prefix (object: POINTER): INTEGER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_node_value (object: POINTER): INTEGER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_parent_node (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_child_nodes (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_first_child (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_last_child (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_previous_sibling (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_next_sibling (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_get_owner_document (object: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_clone_node (object: POINTER; deep: BOOLEAN): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_insert_before (object, new_child, ref_child: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_replace_child (object, new_child, ref_child: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_remove_child (object, old_child: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_append_child (object, new_child: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	dn_has_child_nodes (object: POINTER): INTEGER is

		external
			"C | %"dom_support.h%""
		end

	dn_set_node_value (object, value: POINTER) is

		external
			"C | %"dom_support.h%""
		end

	dn_get_node_type (object: POINTER): INTEGER is

		external
			"C | %"dom_support.h%""
		end

end

