#include <xalanc/Include/PlatformDefinitions.hpp>
// #include <xalanc/Include/XalanVector.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>

#include <xalanc/XalanDOM/XalanDOMString.hpp>
#include <xalanc/PlatformSupport/XalanStdOutputStream.hpp>
#include <xalanc/XercesParserLiaison/XercesDOMFormatterWalker.hpp>
#include <xalanc/XMLSupport/FormatterToXML.hpp>
#include <xalanc/PlatformSupport/XalanOutputStreamPrintWriter.hpp>

#include <string.h>
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

extern "C" {
#include "dom_support.h"
}

XALAN_USING_XERCES(DOMNode)
XALAN_USING_XERCES(XMLString)

XALAN_USING_XALAN(XalanDOMString)
XALAN_USING_XALAN(CharVectorType)

#include "to_unicode.h"

XALAN_USING_XERCES (DOMImplementation)
extern DOMImplementation* impl;

const char *dom_buf_id = "dom_memory_buf";

CharVectorType *dtb;
// stringstream *document_stream = 0; // global DOMDocument output buffer

extern "C" {

// DOMNode

void copy_transcode_buffer (void *dst, int nbytes) {

	memcpy (dst, dtb->begin (), nbytes);
	delete dtb;
}

int dom_store_chars (const XMLCh *utf) {

	XALAN_USING_XALAN(XalanDOMChar)

	if (utf == NULL) {
		return -1;
	} else {
		XalanDOMString xds ((XalanDOMChar*) utf);
		dtb = new CharVectorType;
		xds.transcode (*dtb);
		return dtb->end () - dtb->begin ();
	}
}

int dn_get_node_name (void *o) {

	return dom_store_chars (((DOMNode*) o)->getNodeName ());
}

int dn_get_node_prefix (void *o) {

	return dom_store_chars (((DOMNode*) o)->getPrefix ());
}

int dn_get_node_value (void *o) {

	return dom_store_chars (((DOMNode*) o)->getNodeValue ());
}

int dn_get_node_type (void *o) {

	return ((DOMNode*) o)->getNodeType ();
}

void *dn_get_parent_node (void *o) {

	return ((DOMNode*) o)->getParentNode ();
}

void *dn_get_child_nodes (void *o) {

	return ((DOMNode*) o)->getChildNodes ();
}

void *dn_get_first_child (void *o) {

	return ((DOMNode*) o)->getFirstChild ();
}

void *dn_get_last_child (void *o) {

	return ((DOMNode*) o)->getLastChild ();
}

void *dn_get_previous_sibling (void *o) {

	return ((DOMNode*) o)->getPreviousSibling ();
}

void *dn_get_next_sibling (void *o) {

	return ((DOMNode*) o)->getNextSibling ();
}

void *dn_get_owner_document (void *o) {

	return ((DOMNode*) o)->getOwnerDocument ();
}

void *dn_clone_node (void *o, char deep) {

	return ((DOMNode*) o)->cloneNode (deep);
}

void *dn_insert_before (void *o, void *new_child, void *ref_child) {

	return ((DOMNode*) o)->insertBefore ((DOMNode*) new_child, (DOMNode*) ref_child);
}

void *dn_replace_child (void *o, void *new_child, void *old_child) {

	return ((DOMNode*) o)->replaceChild ((DOMNode*) new_child, (DOMNode*) old_child);
}

void *dn_remove_child (void *o, void *old_child) {

	return ((DOMNode*) o)->removeChild ((DOMNode*) old_child);
}

void *dn_append_child (void *o, void *new_child) {

	return ((DOMNode*) o)->appendChild ((DOMNode*) new_child);
}

int dn_has_child_nodes (void *o) {

	return ((DOMNode*) o)->hasChildNodes ();
}

void dn_set_node_value (void *o, char *value) {

	((DOMNode*) o)->setNodeValue (TU (value));
}

// DOMElement

int de_get_tag_name (void *o) {

	XALAN_USING_XERCES(DOMElement)

	return dom_store_chars (((DOMElement*) o)->getTagName ());
}

int de_get_attribute (void *o, char *name) {

	XALAN_USING_XERCES(DOMElement)

	return dom_store_chars (((DOMElement*) o)->getAttribute (TU (name)));
}

void *de_get_attributes (void *o) {

	XALAN_USING_XERCES(DOMElement)

	return ((DOMElement*) o)->getAttributes ();
}

void *de_get_elements_by_tag_name (void *o, char *tagname) {  // return DOMNodeList

	XALAN_USING_XERCES(DOMElement)

	return ((DOMElement*) o)->getElementsByTagName (TU (tagname));
}

void de_set_attribute (void *o, char *name, char *value) {

	XALAN_USING_XERCES(DOMElement)

	((DOMElement*) o)->setAttribute (TU (name), TU (value));
}

void de_remove_attribute (void *o, char *name) {

	XALAN_USING_XERCES(DOMElement)

	((DOMElement*) o)->removeAttribute (TU (name));
}

// DOMNodeList

void *dnl_item (void *object, int index) {

	XALAN_USING_XERCES(DOMNodeList)

	return ((DOMNodeList*) object)->item (index);
}

int dnl_get_length (void *object) {

	XALAN_USING_XERCES(DOMNodeList)

	return ((DOMNodeList*) object)->getLength ();
}

// DOMNamedNodeMap

void dnnm_delete (void *object) {

	XALAN_USING_XERCES(DOMNamedNodeMap)

	delete (DOMNamedNodeMap*) object;
}

void *dnnm_item (void *o, int index) {

	// gets *DOMNode

	XALAN_USING_XERCES(DOMNamedNodeMap)

	return ((DOMNamedNodeMap*) o)->item ((XMLSize_t) index);
}

int dnnm_length (void *o) {

	XALAN_USING_XERCES(DOMNamedNodeMap)

	return (int) ((DOMNamedNodeMap*) o)->getLength ();
}

// ErrorHandler

void *eh_create () {

	XALAN_USING_XERCES(ErrorHandler)
	XALAN_USING_XERCES(HandlerBase)

	return ((ErrorHandler*) new HandlerBase ());
}

void eh_delete (void *o) {

	XALAN_USING_XERCES(ErrorHandler)

	delete (ErrorHandler*) o;
}

// XercesDOMParser

void *xdp_create () {

	XALAN_USING_XERCES(XercesDOMParser)

	return new XercesDOMParser ();
}

void xdp_delete (void *o) {

	XALAN_USING_XERCES(XercesDOMParser)

	delete (XercesDOMParser*) o;
}

void xdp_set_validation_scheme (void *o, int scheme) {

	XALAN_USING_XERCES(XercesDOMParser)

	((XercesDOMParser*) o)->setValidationScheme ((xercesc_3_1::AbstractDOMParser::ValSchemes) scheme); 
}

void xdp_set_do_schema (void *o, char flag) {

	XALAN_USING_XERCES(XercesDOMParser)

	((XercesDOMParser*) o)->setDoSchema ((bool) flag);
}

void xdp_set_do_namespaces (void *o, char flag) {

	XALAN_USING_XERCES(XercesDOMParser)

	((XercesDOMParser*) o)->setDoNamespaces ((bool) flag);
}

void xdl_set_include_ignorable_whitespace (void *o, char flag) {

	XALAN_USING_XERCES(XercesDOMParser)

	((XercesDOMParser*) o)->setIncludeIgnorableWhitespace ((bool) flag);
}

void xdl_set_error_handler (void *o, void *handler) {

	XALAN_USING_XERCES(XercesDOMParser)
	XALAN_USING_XERCES(ErrorHandler)

	((XercesDOMParser*) o)->setErrorHandler ((ErrorHandler*) handler);
}

void *parse_dom_file (void *o, char *filename) {

	XALAN_USING_XERCES(XercesDOMParser)
	XALAN_USING_XERCES(DOMDocument)
	XALAN_USING_XERCES(XMLException)
	XALAN_USING_XERCES(XMLString)
	XALAN_USING_XERCES(DOMException)
	XALAN_USING_XERCES(SAXParseException)

	DOMDocument *doc;
	int err = 0;

	try {
		((XercesDOMParser*) o)->parse (filename);
	}

	catch (const SAXParseException& toCatch) {
		cerr << "Parse exception at line "
		<< toCatch.getLineNumber ()
		<< " column "
		<< toCatch.getColumnNumber ()
		<< "\n";
		err = 1;
	}

	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		cerr << "XMLException: message is: \n"
			<< message << "\n";
		XMLString::release(&message);
		err = 1;
	}
	catch (const DOMException& toCatch) {
            char* message = XMLString::transcode(toCatch.msg);
            cerr << "DOMException: message is: \n"
                 << message << "\n";
            XMLString::release(&message);
		err = 1;
	}
	catch (...) {
		cerr << "Unexpected Exception.\n" ;
		err = 1;
	}

	return err ? NULL : ((XercesDOMParser*) o)->adoptDocument ();
}

void *parse_dom_mem (void *o, const char *buffer, int buffer_size) {

	XALAN_USING_XERCES(MemBufInputSource)
	XALAN_USING_XERCES(XercesDOMParser)

	const MemBufInputSource theInputSource ((XMLByte*) buffer, buffer_size, dom_buf_id);

	((XercesDOMParser*) o)->parse (theInputSource);

	return ((XercesDOMParser*) o)->getDocument ();
}

// std C++

void *ofstream_create (char *file_name) {

	return new ofstream (file_name);
}

void ofstream_delete (void *o) {

	ofstream* ofs = (ofstream*) o;

	ofs->close ();
	delete ofs;
}

void *stringstream_create () {

	return new stringstream;
}

void stringstream_delete (void *o) {

	delete (stringstream*) o;
}

int stringstream_output_count (void *o) {

	stringstream* ss = (stringstream*) o;

	ss->seekp (0, ios::end);
	return ss->tellp ();
}

void stringstream_move_output (void *o, char *buffer) {

	stringstream* ss = (stringstream*) o;
	int len;

	ss->seekp (0, ios_base::end);
	len = ss->tellp ();
	ss->seekp (0, ios_base::beg);
	ss->read (buffer, len);
}

// XalanStdOutputStream

void *xsos_create (void *stream) { // takes regular C++ stream as argument

	XALAN_USING_XALAN(XalanStdOutputStream)

	return new XalanStdOutputStream (*((ostream*) stream));
}

void xsos_delete (void *o) {

	XALAN_USING_XALAN(XalanStdOutputStream)

	delete (XalanStdOutputStream*) o;
}

// XalanOutputStreamPrintWriter

void *xospw_create (void *xsos) {

	XALAN_USING_XALAN(XalanOutputStreamPrintWriter)
	XALAN_USING_XALAN(XalanStdOutputStream)

	return new XalanOutputStreamPrintWriter(*((XalanStdOutputStream*) xsos));
}

void xospw_delete (void *o) {

	XALAN_USING_XALAN(XalanOutputStreamPrintWriter)

	delete (XalanOutputStreamPrintWriter*) o;
}

// FormatterToXML

void *ftx_create (void *xospw) {

	XALAN_USING_XALAN(FormatterToXML)
	XALAN_USING_XALAN(XalanOutputStreamPrintWriter)

	return new FormatterToXML(*((XalanOutputStreamPrintWriter*) xospw));
}

void ftx_delete (void *o) {

	XALAN_USING_XALAN(FormatterToXML)

	delete (FormatterToXML*) o;
}

void ftx_set_do_indent (void *o, char flag) {

	XALAN_USING_XALAN(FormatterToXML)

	((FormatterToXML*) o)->setDoIndent ((bool) flag);
}

void ftx_set_indent (void *o, int indent) {

	XALAN_USING_XALAN(FormatterToXML)

	((FormatterToXML*) o)->setIndent (indent);
}

void ftx_serialize (void *o, void *document) {

	XALAN_USING_XERCES(XMLString)

	XALAN_USING_XALAN(XercesDOMFormatterWalker)
	XALAN_USING_XALAN(FormatterToXML)
	XALAN_USING_XERCES(DOMDocument)

//	XALAN_USING_XERCES(DOMException)
	XALAN_USING_XERCES(SAXException)

	XercesDOMFormatterWalker theWalker (*((FormatterToXML*) o));
	char *exception_msg;

	try {

		theWalker.traverse ((DOMDocument*) document);

	} catch (const SAXException& toCatch) {

		const int errmax = 200;

		exception_msg = (char*) malloc (errmax);
		if (!XMLString::transcode (toCatch.getMessage (), exception_msg, errmax)) {
			cerr << "Transcode error" << endl;
		}
		cerr << "SAXException " << exception_msg << endl;
		free (exception_msg);
	}		
}

// DOMDocument

void *new_dom_doc (char *qualified_name, char *public_id, char *system_id) {

	XALAN_USING_XERCES(DOMImplementation)
	XALAN_USING_XERCES(DOMDocument)
	XALAN_USING_XERCES(DOMDocumentType)

	DOMDocument* doc;

	if (system_id) {

		DOMImplementation *di = DOMImplementation::getImplementation ();
		DOMDocumentType *dt;

//		cout << "system_id is " << system_id << endl;
		dt = di->createDocumentType (TU (qualified_name), TU (public_id), TU (system_id));
		doc = di->createDocument (0, TU (qualified_name), dt);
	} else {
		if (qualified_name) {
			doc = impl->createDocument (0, TU (qualified_name), 0);
		} else {
			doc = impl->createDocument ();
		}
	}
	// namespace URI, qualified name, DTD

	return doc;
}

void release_dom_doc (void *doc) {

	XALAN_USING_XERCES(DOMDocument)

	((DOMDocument*)doc)->release ();

}

void *dd_create_element (void *o, char *tag_name) {

	XALAN_USING_XERCES(DOMDocument)

	return ((DOMDocument*) o)->createElement (TU (tag_name));
}

void *dd_create_text_node (void *o, char *utf8) {

	XALAN_USING_XERCES(DOMDocument)

	return ((DOMDocument*) o)->createTextNode (TU (utf8));
}

void *dd_create_utf16_text_node (void *o, void *utf16) {

	// Do I have to use c_str () ?

	XALAN_USING_XERCES(DOMDocument)
	XALAN_USING_XALAN(XalanDOMString)
	
	return ((DOMDocument*) o)->createTextNode (
		((XalanDOMString*) utf16)->c_str ()
	);
}

void *dd_import_node (void *o, void* node, char deep) {

	XALAN_USING_XERCES(DOMDocument)

	return ((DOMDocument*) o)->importNode ((DOMNode*) node, (bool) deep);
}

void *dd_get_document_element (void *o) {

	XALAN_USING_XERCES(DOMDocument)

	return ((DOMDocument*) o)->getDocumentElement ();
}

}

