#ifndef DOM_SUPPORT_H
#define DOM_SUPPORT_H

// Code is compiled as C++ but "C" headers are exported

// DOMNode
 
void copy_transcode_buffer (void *dst, int nbytes);
int dn_get_node_name (void *o);
int dn_get_node_prefix (void *o);
int dn_get_node_value (void *o);

int dn_get_node_type (void *o);
void *dn_get_parent_node (void *o);
void *dn_get_child_nodes (void *o);
void *dn_get_first_child (void *o);
void *dn_get_last_child (void *o);
void *dn_get_previous_sibling (void *o);
void *dn_get_next_sibling (void *o);
void *dn_get_owner_document (void *o);
void *dn_clone_node (void *o, char deep);
void *dn_insert_before (void *o, void *new_child, void *ref_child);
void *dn_replace_child (void *o, void *new_child, void *old_child);
void *dn_remove_child (void *o, void *old_child);
void *dn_append_child (void *o, void *new_child);
int dn_has_child_nodes (void *o);
void dn_set_node_value (void *o, char *value);

// DOMElement

int de_get_tag_name (void *o);
int de_get_attribute (void *o, char *name);

void *de_get_attributes (void *o);
void *de_get_elements_by_tag_name (void *o, char *tag_name); // return DOMNodeList
void de_set_attribute (void *o, char *name, char *value);
void de_remove_attribute (void *o, char *name);

// DOMNodeList

void *dnl_item (void *object, int index);
int dnl_get_length (void *object);

// DOMNamedNodeMap

void dnnm_delete (void *object);
void *dnnm_item (void *o, int index); // returns *DOMNode
int dnnm_length (void *o);

// ErrorHandler

void *eh_create ();
void eh_delete (void *o);

// XercesDOMParser

void *xdp_create ();
void xdp_delete (void *o);
void xdp_set_validation_scheme (void *o, int scheme);
void xdp_set_do_schema (void *o, char flag);
void xdl_set_include_ignorable_whitespace (void *o, char flag);
void xdl_set_error_handler (void *o, void *handler);

void *parse_dom_file (void *o, char *filename);
void *parse_dom_mem (void *o, const char *buffer, int buffer_size);

// std C++

void *ofstream_create (char *file_name);
void ofstream_delete (void *o);
void *stringstream_create ();
void stringstream_delete (void *o);
int stringstream_output_count (void *o);
void stringstream_move_output (void *o, char *buffer);

// XalanStdOutputStream

void *xsos_create (void *stream); // takes regular C++ stream as argument
void xsos_delete (void *o);

// XalanOutputStreamPrintWriter

void *xospw_create (void *xsos);
void xospw_delete (void *o);

// FormatterToXML

void *ftx_create (void *xospw);
void ftx_delete (void *o);
void ftx_set_do_indent (void *o, char flag);
void ftx_set_indent (void *o, int indent);
void ftx_serialize (void *o, void *document);

// DOMDocument

void *new_dom_doc (char *qualified_name, char *public_id, char *system_id);
void release_dom_doc (void *doc);

void *dd_create_element (void *o, char *tag_name);
void *dd_create_text_node (void *o, char *utf8);
void *dd_create_utf16_text_node (void *o, void *utf16);
void *dd_import_node (void *o, void* node, char deep); // import DOMNode
void *dd_get_document_element (void *o);
#endif
