XALAN_USING_XALAN(Function)
XALAN_USING_XALAN(XPathExecutionContext)
XALAN_USING_XALAN(XalanDOMString)
XALAN_USING_XALAN(XalanNode)
XALAN_USING_XALAN(XObjectPtr)
XALAN_USING_XALAN(MemoryManager)
XALAN_USING_XALAN(XalanCopyConstruct)
// XALAN_USING_XALAN(XalanNamedNodeMap)

class FunctionAttrCount : public Function {
public:
	virtual XObjectPtr execute(
		XPathExecutionContext&          executionContext,
		XalanNode*                      context,
		const XObjectArgVectorType&     args,
		const LocatorType*              locator) const;

	using Function::execute;

	virtual FunctionAttrCount* clone (MemoryManager& theManager) const {

		return XalanCopyConstruct (theManager, *this);
	}

protected:

	const XalanDOMString& getError (XalanDOMString& theResult) const {

		theResult.assign("attr-count takes no arguments");
		return theResult;
	}
/*
private:
	// Not implemented...
	FunctionAttrCount& operator=(const FunctionAttrCount&);
	bool operator==(const FunctionAttrCount&) const;
*/
};

class FunctionAttrName : public Function {
public:
	virtual XObjectPtr execute(
		XPathExecutionContext&          executionContext,
		XalanNode*                      context,
		const XObjectArgVectorType&     args,
		const LocatorType*              locator) const;

	using Function::execute;

	virtual FunctionAttrName* clone (MemoryManager& theManager) const {

		return XalanCopyConstruct (theManager, *this);
	}

protected:

	const XalanDOMString& getError (XalanDOMString& theResult) const {

		theResult.assign("bad attr-name argument");
		return theResult;
	}
};

class FunctionAttrValue : public Function {
public:
	virtual XObjectPtr execute(
		XPathExecutionContext&          executionContext,
		XalanNode*                      context,
		const XObjectArgVectorType&     args,
		const LocatorType*              locator) const;

	using Function::execute;

	virtual FunctionAttrValue* clone (MemoryManager& theManager) const {

		return XalanCopyConstruct (theManager, *this);
	}

protected:

	const XalanDOMString& getError (XalanDOMString& theResult) const {

		theResult.assign("bad attr-value argument");
		return theResult;
	}
};

XObjectPtr FunctionAttrCount::execute(
	XPathExecutionContext&          executionContext,
	XalanNode*                      context,
	const XObjectArgVectorType&     args,
	const LocatorType*              locator) const {

	if (args.size () != 0){
		XPathExecutionContext::GetAndReleaseCachedString theGuard (executionContext);
            generalError (executionContext, context, locator);
	}

	return executionContext.getXObjectFactory().createNumber (
		context->getAttributes ()->getLength()
	);
}

XObjectPtr FunctionAttrName::execute(
	XPathExecutionContext&          executionContext,
	XalanNode*                      context,
	const XObjectArgVectorType&     args,
	const LocatorType*              locator) const {

	if (args.size () != 1){
		XPathExecutionContext::GetAndReleaseCachedString theGuard (executionContext);
            generalError (executionContext, context, locator);
	}

	int index = (int) args[0]->num (executionContext);
	int max = context->getAttributes ()->getLength();

	if (index < 1 || index > max) {
		XPathExecutionContext::GetAndReleaseCachedString theGuard (executionContext);
            generalError (executionContext, context, locator);
	}

	XalanNode* node = context->getAttributes ()->item (index - 1);

	return executionContext.getXObjectFactory().createString (
		node->getNodeName ()
	);
}

XObjectPtr FunctionAttrValue::execute(
	XPathExecutionContext&          executionContext,
	XalanNode*                      context,
	const XObjectArgVectorType&     args,
	const LocatorType*              locator) const {

	if (args.size () != 1){
		XPathExecutionContext::GetAndReleaseCachedString theGuard (executionContext);
            generalError (executionContext, context, locator);
	}

	int index = (int) args[0]->num (executionContext);
	int max = context->getAttributes ()->getLength();

	if (index < 1 || index > max) {
		XPathExecutionContext::GetAndReleaseCachedString theGuard (executionContext);
            generalError (executionContext, context, locator);
	}

	XalanNode* node = context->getAttributes ()->item (index - 1);

	return executionContext.getXObjectFactory().createString (
		node->getNodeValue ()
	);
}

