indexing
	description:
		"Xalan document interface"
		"the library must exist because it initialises xstpl"

class
	XALAN_DOCUMENT

inherit
	XML
	XALAN_NODE
		rename
			make as xalan_node_make
		end
	DISPOSABLE

create
	make, make_from_file, make_from_string, make_from_xerces, make_from_object

feature -- Creation

	make, make_from_file (filename: STRING) is

		do
			common_make
			obj := parse_xml_file (terminated (filename))
		end

	make_from_string (xml_data: STRING) is

		do
			common_make
			obj := parse_xml_mem (terminated (xml_data), xml_data.count)
		end

	make_from_xerces (xerces_doc: DOM_DOCUMENT) is

		do
			common_make
			parser_liaison := new_parser_liaison
			obj := wrap_xerces_doc (xerces_doc.obj, parser_liaison)
		end

	make_from_object (object: POINTER) is
			--
		do
			common_make
			xalan_node_make (object)
		end

	dispose is
			-- Dispose of the document if no Xerces parser liaison exists.
			-- Xerces parser liaison owns the document, so don't delete it
		do
			if has_xpaths then
				dispose_xpaths
			end
			if parser_liaison  /= default_pointer then
				delete_parser_liaison (parser_liaison )
				parser_liaison := default_pointer
				obj := default_pointer
			end
			if obj /= default_pointer then
				delete_xml_doc (obj)
				obj := default_pointer
			end
		end

	has_xpaths: BOOLEAN is
			--
		do
			Result := xpath_refs /= Void and then xpath_refs.count > 0
		end

	dispose_xpaths is

		local
			cursor: PCURSOR[XPATH]
		do
			if xpath_refs /= Void then
				from cursor := xpath_refs.new_cursor
				until cursor.after
				loop
					cursor.item.dispose
					cursor.forth
				end
				xpath_refs := Void
			end
		end

feature -- Access

	new_xpath (context: STRING): XPATH is
			-- Keep a reference to the object
		do
			create Result.xpath_make (Current)
			add_reference (Result)
			if context /= Void then
				Result.set_context (context)
			end
		end

	document_element: XALAN_ELEMENT is
			--
		do
			create Result.make (xalan_root_element (obj))
		end

feature {XPATH} -- XPATH features

	add_reference (xp: XPATH) is

		do
			xpath_refs.put_last (xp)
		end

	remove_reference (xp: XPATH) is

		local
			cursor: PCURSOR[XPATH]
			found: BOOLEAN
		do
			from cursor := xpath_refs.new_cursor
			until found or cursor.after
			loop
				if cursor.item = xp then
					cursor.remove
					found := true
				else
					cursor.forth
				end
			end
		end

feature {} -- Private

	xpath_refs: PLIST[XPATH]
	parser_liaison: POINTER

	common_make is
			--
		do
			xml.initialize
			create xpath_refs.make
		end

feature {} -- External

	new_parser_liaison: POINTER is

		external
			"C | %"xalan_support.h%""
		end

	delete_parser_liaison (pl: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	wrap_xerces_doc (xerces_dom, pl: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	parse_xml_file (filename: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	parse_xml_mem (buffer: POINTER; buffer_size: INTEGER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xalan_root_element (doc: POINTER): POINTER is
			-- returns *XalanElement
		external
			"C | %"xalan_support.h%""
		end

	delete_xml_doc (doc: POINTER) is
			-- delete XalanDocument
		external
			"C | %"xalan_support.h%""
		end

end
