indexing
	description:
		"Interface to XalanDOMString"

class
	XALAN_DOM_STRING

inherit
	UTF_SUPPORT
		redefine
			is_equal
		end
	CL_MISC
		export
			{} all
		redefine
			is_equal
		end
	DISPOSABLE
		redefine
			is_equal
		end

create
	make, make_empty, make_undefined, make_from_string, make_from_xalan_dom_string

feature -- Creation

	make (object: POINTER) is

		do
			obj := object
			count := xds_length (obj)
		end

	make_empty is

		do
			make_undefined (0)
		end

	make_undefined (char_count: INTEGER) is

		do
			obj := new_xalan_dom_string (char_count)
			count := xds_length (obj)
		end

	make_from_xalan_dom_string (src: XALAN_DOM_STRING) is
			--
		do
			obj := copy_xalan_dom_string (src.obj)
			count := xds_length (obj)
		end

	make_from_string (src: STRING) is
			--
		do
			obj := make_xds_from_string (terminated (src))
			count := xds_length (obj)
		end

	dispose is

		do
			delete_xalan_dom_string (obj)
		end

feature -- Access

	obj: POINTER
	count: INTEGER

	to_string: STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		do
			Result := get_transcoded (xds_transcode (obj))
		end

	infix "<" (other: XALAN_DOM_STRING): BOOLEAN is
			--
		do
			Result := xds_compare (obj, other.obj) < 0
		end

	infix ">" (other: XALAN_DOM_STRING): BOOLEAN is
			--
		do
			Result := xds_compare (obj, other.obj) > 0
		end

	is_equal (other: XALAN_DOM_STRING): BOOLEAN is
			--
		do
			Result := xds_compare (obj, other.obj) = 0
		end

	infix "@" (index: INTEGER): INTEGER is
			-- return utf16 character
		require
			valid_index: index >= 1
		do
			Result := xds_at (obj, index - 1)
		ensure
			integer16 : Result & 0xFFFF0000 = 0
		end

	infix "+" (other: XALAN_DOM_STRING): XALAN_DOM_STRING  is
			--
		do
			create Result.make_from_xalan_dom_string (Current)
			Result.append_string (other)
		end

	append_string (other: XALAN_DOM_STRING) is
			--
		do
			xds_append_string (obj, other.obj)
			count := xds_length (obj)
		end

	put (utf: INTEGER; index: INTEGER) is
			-- set utf16 character
		require
			valid_index: index >= 1
			integer16 : utf & 0xFFFF0000 = 0
		do
			xds_put (obj, utf, index - 1)
		end

	substring (start_index, end_index: INTEGER): XALAN_DOM_STRING is

		require
			sane: start_index <= end_index
		local
			n: INTEGER
			p: POINTER
		do
			n := end_index - start_index + 1
			p := new_xalan_dom_string (n)
			create Result.make (xds_substr (obj, p, start_index - 1, n))
		ensure
			Result.count = end_index - start_index + 1
		end

	substring_index (other: XALAN_DOM_STRING; start_index: INTEGER): INTEGER is
			-- adjust for C indices
		do
			Result := xds_substring_index (obj, other.obj, start_index - 1) + 1
		end

	reverse is

		do
			xds_reverse (obj)
		end

	codes: STRING is

		local
			i: INTEGER
		do
			create Result.make_empty
			from i := 1
			until i > count
			loop
				if i > 1 then
					Result.append_string (", ")
				end
				Result.append_string ((Current @ i).out)
				i := i + 1
			end
		end

feature {} -- Private

	compare (other: XALAN_DOM_STRING): INTEGER is
			--
		do
			Result := xds_compare (obj, other.obj)
		end

feature {} -- Externals

	new_xalan_dom_string (npos: INTEGER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	make_xds_from_string (src: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	copy_xalan_dom_string (src: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	delete_xalan_dom_string (object: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	xds_length (object: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	xds_substr (object, dst: POINTER; posn, n: INTEGER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xds_transcode (object: POINTER): INTEGER is
			--
		external
			"C | %"xalan_support.h%""
		end

	xds_at (object: POINTER; index: INTEGER): INTEGER is
			-- return utf16 character
		external
			"C | %"xalan_support.h%""
		end

	xds_compare (object, other: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	xds_substring_index (object, other: POINTER; start_index: INTEGER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	xds_append_string (object, other: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	xds_put (object: POINTER; utf16, index: INTEGER) is
			-- set utf16 character
		external
			"C | %"xalan_support.h%""
		end

	xds_reverse (object: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

end
