indexing
	description:
		"Xalan Node interface, uses terminated() for string reads"

class
	XALAN_NODE

inherit
	XML
	NODE_TYPES

create
	make

feature -- Creation

	make (object: POINTER) is

		do
			obj := object
		end

feature -- Access

	obj: POINTER

	attr (name: STRING): STRING is
			--
		local
			attr_node: XALAN_NODE
		do
			attr_node := attributes.named_item (name)
			if attr_node /= Void then
				Result := attr_node.node_value
			end
		end

	attr_utf (name: STRING): XALAN_DOM_STRING is
			--
		local
			attr_node: XALAN_NODE
		do
			attr_node := attributes.named_item (name)
			if attr_node /= Void then
				Result := attr_node.node_value_utf
			end
		end

	node_name: STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		do
			Result := get_transcoded (xn_get_node_name (obj))
		end

	node_value: STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		do
			Result := get_transcoded (xn_get_node_value (obj))
		end

	node_value_utf: XALAN_DOM_STRING is
			-- The Raw node value is owned by the node, don't free it.
		do
			create Result.make (xn_get_node_value_utf (obj))
		end

	node_type: INTEGER is

		do
			Result := get_xalan_node_type (obj)
		end

	has_owner (document: XALAN_DOCUMENT): BOOLEAN is
			--
		do
			Result := document.obj = xn_get_owner_document (obj)
		end

	parent_node: XALAN_NODE is

		do
			Result := xml.create_xalan_node (xn_get_parent_node (obj))
		end

	child_nodes: XALAN_NODE_LIST is

		local
			p: POINTER
		do
			p := xn_get_child_nodes (obj)
			if p /= default_pointer then
				create Result.make (p)
			end
		end

	first_child: XALAN_NODE is

		do
			Result := xml.create_xalan_node (xn_get_first_child (obj))
		end

	last_child: XALAN_NODE is

		do
			Result := xml.create_xalan_node (xn_get_last_child (obj))
		end

	previous_sibling: XALAN_NODE is

		do
			Result := xml.create_xalan_node (xn_get_previous_sibling (obj))
		end

	next_sibling: XALAN_NODE is

		do
			Result := xml.create_xalan_node (xn_get_next_sibling (obj))
		end

	attributes: XALAN_NAMED_NODE_MAP is
			--
		do
			create Result.make (xn_get_attributes (obj))
		end

	has_child_nodes: BOOLEAN is
			--
		do
			Result := xn_get_first_child (obj) /= default_pointer
		end

	dom_node: DOM_NODE is
			--
		do
			Result := xml.create_dom_node (xn_map_node (obj))
		end

feature {} -- Externals

	get_xalan_node_type (object: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		alias
			"xn_get_node_type"
		end

	xn_get_node_name (object: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_node_value (object: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_node_value_utf (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_owner_document (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_parent_node (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_child_nodes (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_first_child (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_last_child (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_previous_sibling (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_next_sibling (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_get_attributes (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xn_map_node (object: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

end

