#include <xalanc/Include/PlatformDefinitions.hpp>

#if defined(XALAN_CLASSIC_IOSTREAMS)
#include <iostream.h>
#else
#include <iostream>
#endif

#include <sstream>

#include <xercesc/util/XMLString.hpp>
#include "to_unicode.h"

#include <xercesc/sax/EntityResolver.hpp>
#include <xercesc/sax/InputSource.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>

#include <xercesc/parsers/XercesDOMParser.hpp>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOMImplementation.hpp>

#include <xalanc/DOMSupport/XalanDocumentPrefixResolver.hpp>
#include <xalanc/XPath/XPathEvaluator.hpp>
#include <xalanc/XPath/NodeRefList.hpp>
#include <xalanc/XPath/XPathParserException.hpp>
#include <xalanc/XalanSourceTree/XalanSourceTreeParserLiaison.hpp>

#include <xalanc/XalanTransformer/XalanTransformer.hpp>
#include <xalanc/XalanTransformer/XercesDOMWrapperParsedSource.hpp>
#include <xalanc/XalanSourceTree/XalanSourceTreeDOMSupport.hpp>
#include <xalanc/XalanSourceTree/XalanSourceTreeInit.hpp>
#include <xalanc/XalanSourceTree/XalanSourceTreeParserLiaison.hpp>
#include <xalanc/XercesParserLiaison/FormatterToXercesDOM.hpp>
#include <xalanc/XercesParserLiaison/XercesDocumentWrapper.hpp>
#include <xalanc/XalanTransformer/XalanCompiledStylesheet.hpp>
#include <xalanc/XalanDOM/XalanDOMString.hpp>
#include <xalanc/XalanDOM/XalanElement.hpp>
#include <xalanc/XalanDOM/XalanNode.hpp>
#include <xalanc/XalanDOM/XalanNodeList.hpp>
#include <xalanc/XalanDOM/XalanNamedNodeMap.hpp>

#include <xalanc/PlatformSupport/XalanOutputStreamPrintWriter.hpp>
#include <xalanc/PlatformSupport/XalanStdOutputStream.hpp>

#include <xalanc/XMLSupport/FormatterToXML.hpp>
#include <xalanc/XMLSupport/FormatterTreeWalker.hpp>
#include <xalanc/XPath/XObject.hpp>

#include <assert.h>

// Includes for utf8

#include <sstream>
#include <string.h>

extern "C" {
#include "xalan_support.h"
}

// XALAN_USING_XERCES(XMLString)
XALAN_USING_XALAN(XalanDOMString)

XALAN_USING_XALAN(XalanTransformer)
XALAN_USING_XALAN(XSLTInputSource)
XALAN_USING_XALAN(XalanCompiledStylesheet)
XALAN_USING_XALAN(XalanParsedSource)
XALAN_USING_XALAN(XSLTResultTarget)

XALAN_USING_XERCES(XMLPlatformUtils)
XALAN_USING_XERCES(DOMImplementation)
XALAN_USING_XERCES(DOMDocumentType)
XALAN_USING_XERCES(DOMDocument)
XALAN_USING_XERCES(DOMNode)
XALAN_USING_XERCES(DOMElement)

XALAN_USING_XALAN(XercesDOMSupport)
XALAN_USING_XALAN(XercesParserLiaison)
XALAN_USING_XALAN(XercesDOMWrapperParsedSource)

XALAN_USING_XERCES(EntityResolver)
XALAN_USING_XERCES(InputSource)
XALAN_USING_XERCES(LocalFileInputSource)
XALAN_USING_XALAN(XSLTResultTarget)
XALAN_USING_XALAN(XalanNode)
XALAN_USING_XALAN(CharVectorType)
XALAN_USING_XALAN(XalanSourceTreeDOMSupport)
XALAN_USING_XALAN(XalanSourceTreeParserLiaison)

// transform modes, see xalan_transformer.e

#define SRC_FILE 1
#define SRC_STRING 2
#define SRC_XERCES 4
#define DST_FILE 8
#define DST_STRING 16
#define DST_XERCES 32
#define XSL_FILE 64
#define XSL_CSS 128

using namespace std;

// Globals

DOMImplementation *impl;
XalanSourceTreeDOMSupport *xstds;
XalanSourceTreeParserLiaison *xstpl;

// Dynamically allocated

XALAN_USING_XALAN(XercesDOMSupport)
XercesDOMSupport *xds = 0;

XALAN_USING_XALAN(XercesParserLiaison) 
XercesParserLiaison *xpl = 0;

XALAN_USING_XALAN(XalanDOMString)
XalanDOMString *efn = 0; // external function namespace

extern CharVectorType *dtb;

extern "C" {

// XalanNode

int xn_store_chars (const XalanDOMString xds) {

	if (xds.begin () == NULL) {
		return -1;
	} else {
		dtb = new CharVectorType;
		xds.transcode (*dtb);
		return dtb->end () - dtb->begin ();
	}
}

int xn_get_node_name (void *o) {

	return xn_store_chars (((XalanNode*) o)->getNodeName ());
}

int xn_get_node_value (void *o) {

	return xn_store_chars (((XalanNode*) o)->getNodeValue ());
}

void *xn_get_node_value_utf (void *o) {

	// returned node value is owned by XalanNode
	// copy it so that it can be disposed like the others

	XALAN_USING_XALAN(XalanDOMString)

	const XalanDOMString *xds = &((XalanNode*) o)->getNodeValue ();

	return (void*) new XalanDOMString (*xds);
}

int xn_get_node_type (void *o) {

	return ((XalanNode*) o)->getNodeType ();
}

void *xn_get_owner_document (void *o) { // returns *XalanDocument

	return (void*) ((XalanNode*) o)->getOwnerDocument ();
}

void *xn_get_parent_node (void *o) {

	return (void*) ((XalanNode*) o)->getParentNode ();
}

void *xn_get_child_nodes (void *o) {

	return (void*) ((XalanNode*) o)->getChildNodes ();
}

void *xn_get_first_child (void *o) {

	return (void*) ((XalanNode*) o)->getFirstChild ();
}

void *xn_get_last_child (void *o) {

	return (void*) ((XalanNode*) o)->getLastChild ();
}

void *xn_get_previous_sibling (void *o) {

	return (void*) ((XalanNode*) o)->getPreviousSibling ();
}

void *xn_get_next_sibling (void *o) {

	return (void*) ((XalanNode*) o)->getNextSibling ();
}

void *xn_get_attributes (void *o) {

	return (void*) ((XalanNode*) o)->getAttributes ();
}

void *xn_map_node (void *o) {

	XALAN_USING_XALAN(XalanDocument)
	XALAN_USING_XALAN(XercesDocumentWrapper)
	XALAN_USING_XALAN(XalanNode)

	XalanDocument *doc = ((XalanNode*) o)->getOwnerDocument ();

	return const_cast<DOMNode*> (((XercesDocumentWrapper*) doc)->mapNode ((XalanNode*) o));
}

// XalanElement

int xe_get_tag_name (void *o) {

	XALAN_USING_XALAN(XalanElement)
	return xn_store_chars (((XalanElement*) o)->getTagName ());
}

// XalanTransformer

void initialize () {

	XMLPlatformUtils::Initialize();
	XalanTransformer::initialize();
}

void terminate () {

	XalanTransformer::terminate();
	XMLPlatformUtils::Terminate();
	XalanTransformer::ICUCleanUp();
}

void *new_transformer () {

	XalanTransformer *xt = new XalanTransformer;
	return xt;
}

void delete_transformer (void *t) {

	delete (XalanTransformer*) t;
}

int v2 (void *t, char *xml, char *xsl) {

// transformer testing

	XalanTransformer *xt = (XalanTransformer*) t;
	int ss, theResult;
	XSLTInputSource xis (xsl);

	std::stringstream *out_stream;

	out_stream = new stringstream;

	XSLTResultTarget *rt = new XSLTResultTarget (out_stream);
            
//	theTransformer.setUseValidation (true);

	theResult = xt->transform (xml, xis, *rt);

	if (theResult == 0) {
		out_stream->seekp (0, ios::end);
		ss =  out_stream->tellp ();
		cout << "ss1 = " << ss << endl;
		cout << out_stream->str () ;
	} else {

		cerr << xt->getLastError() << endl;
	}

	delete rt;
	return theResult;
}

int feature_test (void *t, char *xml, char *xsl) {

	cout << "xml = " << xml << endl;
	cout << "xsl = " << xsl << endl;

	return v2 (t, xml, xsl);
}

int transform_xml (int mode, void *t, void *xml, void *xsl, void *res) {

	XALAN_USING_XERCES(DOMDocument)
	XALAN_USING_XALAN(FormatterToXercesDOM)

	XalanTransformer *xt = (XalanTransformer*) t;
	XalanParsedSource *xt_in = (XalanParsedSource*) xml;
	XalanCompiledStylesheet *xt_xsl = (XalanCompiledStylesheet*) xsl;
	XSLTResultTarget *xt_out;

	DOMDocument* doc;
	FormatterToXercesDOM *theFormatter;

	int r;

	if ((mode & (DST_FILE | DST_STRING | DST_XERCES)) == DST_XERCES) {
		doc = (DOMDocument*) res;
		theFormatter = new FormatterToXercesDOM (doc, 0);
		r = xt->transform (*xt_in, xt_xsl, *theFormatter);
		delete theFormatter;
	} else {
		xt_out = (XSLTResultTarget*) res;
		r = xt->transform (*xt_in, xt_xsl, *xt_out);
	}

	if (r != 0) {
		cerr << "transform error: " << xt->getLastError () << endl;
	}

	return r;
}

int stream_length (void *stream) {

	stringstream *ss  = (stringstream*) stream;

	ss->seekg (0, ios::end);
	return ss->tellg ();
}

void copy_stream (void *stream, char *strdata, int length) {

	stringstream *ss  = (stringstream*) stream;
	   
	ss->seekg (0, ios::beg);
	ss->read (strdata, length);
}

void parameter (void *t, char *key, char *expression) {

	XALAN_USING_XALAN(XalanTransformer)

	((XalanTransformer*) t)->setStylesheetParam (key, expression);
}

void xt_set_indent (void *t, int indent) {

	XALAN_USING_XALAN(XalanTransformer)

	((XalanTransformer*) t)->setIndent (indent);
}

void validation (void *t, int flag) {

	XALAN_USING_XALAN(XalanTransformer)

	((XalanTransformer*) t)->setUseValidation (flag);
}

void *parse_file (void *t, char *fname) {

	XALAN_USING_XALAN(XSLTInputSource)
	XALAN_USING_XALAN(XalanParsedSource)
	XALAN_USING_XALAN(XalanTransformer)

	XSLTInputSource is (fname);
	const XalanParsedSource *ps = 0;
	((XalanTransformer*) t)->parseSource (is, ps);
	return (void*) ps;
}

void *parse_string (void *t, char *buffer, int size) {

	XALAN_USING_XALAN(XSLTInputSource)
	XALAN_USING_XALAN(XalanParsedSource)
	XALAN_USING_XALAN(XalanTransformer)

	stringstream ss (ios::in);
	stringbuf* sb;

	sb = ss.rdbuf ();
	sb->pubsetbuf (buffer, size);

	XSLTInputSource is (ss);
	const XalanParsedSource *ps = 0;

	((XalanTransformer*) t)->parseSource (is, ps);
	return (void*) ps;
}

void *dom_wrapper (void *t, void *dom) {

	XALAN_USING_XERCES(DOMDocument)
	XALAN_USING_XALAN(XercesDOMSupport)
	XALAN_USING_XALAN(XercesParserLiaison)
	XALAN_USING_XALAN(XercesDOMWrapperParsedSource)

	if (!xpl) xpl = new XercesParserLiaison; 
	if (!xds) xds = new XercesDOMSupport (*xpl);

	((DOMDocument*) dom)->normalize ();

	const XercesDOMWrapperParsedSource *ps = new XercesDOMWrapperParsedSource (
		((DOMDocument*) dom),
		*xpl,
		*xds
	);

	return (void*) ps;
}

void *compile_stylesheet (void *t, char *fname) {

	const XalanCompiledStylesheet *css = 0;
	XalanTransformer *xt = (XalanTransformer*) t;
	int r = xt->compileStylesheet (fname, css);
	if (r != 0) cerr << "compile error: " << xt->getLastError () << endl;
	return (void*) css;
}

void *string_stream () {

	stringstream *stream = new stringstream;

	return stream;
}

void *string_target (void *stream) {

	XALAN_USING_XALAN(XSLTResultTarget)

	XSLTResultTarget *rt = new XSLTResultTarget ((stringstream*) stream);
	return rt;
}

void *file_target (char *fname) {

	XALAN_USING_XALAN(XSLTResultTarget)

	XSLTResultTarget *rt = new XSLTResultTarget (fname);
	return rt;
}

void delete_stringstream (void *stream) {

	delete (stringstream*) stream;
}

void delete_target (void *target) {

	delete (XSLTResultTarget*) target;
}

// XalanDocument

const char *duplicate_buf_id = "xml_memory_buf"; // xml_support.cc

void *new_parser_liaison () {

	XALAN_USING_XALAN(XercesParserLiaison)

	return (void*) new XercesParserLiaison; 
}

void delete_parser_liaison (void *pl) {

	XALAN_USING_XALAN(XercesParserLiaison)

	XercesParserLiaison *l = (XercesParserLiaison*)pl;

	delete l;
}

void *wrap_xerces_doc (void *xerces_dom, void *pl) {

	XALAN_USING_XALAN(XercesParserLiaison)
	XALAN_USING_XALAN(XalanDocument)
	XALAN_USING_XERCES(DOMDocument)

	XercesParserLiaison *l = (XercesParserLiaison*)pl;
	XalanDocument* doc = l->createDocument ((DOMDocument*) xerces_dom, true, true, true); 

	return (void*) doc;
}

void *parse_xml_file (char *filename) {

	XALAN_USING_XALAN(XalanDOMString)
	XALAN_USING_XERCES(LocalFileInputSource)
	XALAN_USING_XALAN(XalanDocument)

	XALAN_USING_XALAN(XalanSourceTreeParserLiaison)

	const XalanDOMString theFileName (filename);
	const LocalFileInputSource theInputSource (theFileName.c_str());

	XalanDocument* doc = xstpl->parseXMLStream (theInputSource);

	return (void*) doc;
}

void *parse_xml_mem (const char *buffer, int buffer_size) {

	XALAN_USING_XERCES(MemBufInputSource)
	XALAN_USING_XALAN(XalanDocument)

	XALAN_USING_XALAN(XalanSourceTreeParserLiaison)

	const MemBufInputSource theInputSource ((XMLByte*) buffer, buffer_size, duplicate_buf_id);

	XalanDocument* doc = xstpl->parseXMLStream (theInputSource);

	return (void*) doc;
}

void *xalan_root_element (void* doc) {

	// Returns *XalanElement

	XALAN_USING_XALAN(XalanDocument)

	return (void*) ((XalanDocument*)doc)->getDocumentElement ();
}

void delete_xml_doc (void *doc) {

	XALAN_USING_XALAN(XalanDocument)
	XALAN_USING_XALAN(XalanSourceTreeParserLiaison)

	xstpl->destroyDocument ((XalanDocument*) doc);
}

//XPath

int xp_eval (void *pr, void *pe, void *cn, char *expr) {

	XALAN_USING_XALAN(XalanNode)
	XALAN_USING_XALAN(XObjectPtr)
	XALAN_USING_XALAN(XalanDocumentPrefixResolver)
	XALAN_USING_XALAN(XPathEvaluator)

	const XObjectPtr theResult(

		((XPathEvaluator *)pe)->evaluate(
			*xstds,
			(XalanNode*) cn,
			XalanDOMString((char*)expr).c_str(),
			*((XalanDocumentPrefixResolver*) pr)
		)
	);
	assert (theResult.null() == false);

	return xn_store_chars (theResult->str());
}

void *xp_eval_utf16 (void *pr, void *pe, void *cn, char *expr) {

	XALAN_USING_XALAN(XalanNode)
	XALAN_USING_XALAN(XObjectPtr)
	XALAN_USING_XALAN(XalanDocumentPrefixResolver)
	XALAN_USING_XALAN(XPathEvaluator)

	const XObjectPtr theResult(

		((XPathEvaluator *)pe)->evaluate(
			*xstds,
			(XalanNode*) cn,
			XalanDOMString((char*)expr).c_str(),
			*((XalanDocumentPrefixResolver*) pr)
		)
	);
	assert (theResult.null() == false);

	XalanDOMString* s = new XalanDOMString (theResult->str());
	return (void*) s;
}

// NodeRefList
 
void *nrl_get_list (void *pe, void* cn, void *expr, void *doc) {

	XALAN_USING_XALAN(NodeRefList)
	XALAN_USING_XALAN(XPathEvaluator)
	XALAN_USING_XALAN(XalanNode)
	XALAN_USING_XALAN(XalanDOMString)
	XALAN_USING_XALAN(XalanDocument)
	XALAN_USING_XALAN(XPathParserException)

	NodeRefList* theResult = new NodeRefList;

	try {
		((XPathEvaluator*) pe)->selectNodeList (
			*theResult,
			*xstds,
			(XalanNode*) cn,
			XalanDOMString((char*)expr).c_str(),
			((XalanDocument*) doc)->getDocumentElement ()
		);
	} catch (const XPathParserException& toCatch) {
		theResult = NULL;
	}

	return theResult;
}

int nrl_get_length (void *o) {

	XALAN_USING_XALAN(NodeRefList)

	return ((NodeRefList*) o)->getLength ();
}

void *nrl_item (void *o, int index) {

	// returns XalanNode*

	XALAN_USING_XALAN(NodeRefList)

	return (void*) ((NodeRefList*) o)->item (index);
}

void nrl_delete (void *nrl) {

	// Delete the NodeRefList that was created by get_list

	XALAN_USING_XALAN(NodeRefList)

	delete (NodeRefList*) nrl;
}

// XalanNodeList

int xnl_get_length (void *o) {

	XALAN_USING_XALAN(XalanNodeList)

	return ((XalanNodeList*) o)->getLength ();
}

void *xnl_item (void *o, int index) {

	// returns XalanNode*

	XALAN_USING_XALAN(XalanNodeList)

	return (void*) ((XalanNodeList*) o)->item (index);
}

// XalanNamedNodeMap

int xnnm_get_length (void *o) {

	XALAN_USING_XALAN(XalanNamedNodeMap)

	return ((XalanNamedNodeMap*) o)->getLength ();
}

void *xnnm_item (void *o, int index) {

	// returns XalanNode*

	XALAN_USING_XALAN(XalanNamedNodeMap)

	return (void*) ((XalanNamedNodeMap*) o)->item (index);
}

void *xnnm_get_named_item (void *o, char *name) {

	// returns XalanNode*

	XALAN_USING_XALAN(XalanNamedNodeMap)

	return (void*) ((XalanNamedNodeMap*) o)
		->getNamedItem (XalanDOMString (name));
}

// XalanDOMString

void *new_xalan_dom_string (int npos) {

	return (void*) new XalanDOMString (npos, 0);
}

void *make_xds_from_string (void *src) {

	return (void*) new XalanDOMString ((char*) src);
}

void *copy_xalan_dom_string (void *src) {

	return (void*) new XalanDOMString (*((XalanDOMString*) src));
}

void delete_xalan_dom_string (void *object) {

	delete (XalanDOMString*) object;
}

int xds_length (void *o) {

	return ((XalanDOMString*) o)->length ();
}

void *xds_substr (void *o, void *dst, int posn, int n) {

	return (void*) &((XalanDOMString*) o)->substr (
		*((XalanDOMString*) dst),
		posn,
		n
	);
}

int xds_transcode (void *o) {

	return xn_store_chars (*((XalanDOMString*) o));
/*
	XALAN_USING_XALAN(CharVectorType)

	CharVectorType cv;
	((XalanDOMString*) o)->transcode (cv);
	int r = cv.end () - cv.begin ();
//	cout << "cv:" << cv.begin () << endl;
//	cout << "r = " << r << " max = " << max << endl;
	
	if (r > max) {
		r = -1;
	} else {
		memcpy (dst, cv.begin (), r);
	}
	return r;
*/
}

int xds_at (void *o, int index) {

	const XMLCh* src = ((XalanDOMString*) o)->c_str ();

	return (int) src[index];
}

int xds_compare (void *o, void *other) {

	return ((XalanDOMString*) o)->compare (
		*((XalanDOMString*) other)
	);
}

int xds_substring_index (void *o, void* other, int start_index) {

	XALAN_USING_XALAN(CharVectorType)

	XalanDOMString ts = *(XalanDOMString*) o;
	XalanDOMString os = *(XalanDOMString*) other;
/*
	CharVectorType tcv;
	CharVectorType ocv;

	ts.transcode (tcv);
	os.transcode (ocv);
	cout << "this:" << tcv.begin () << " other:" << ocv.begin () << endl;
*/
	int ol = os.length ();
	int nc = ts.length () - ol + 1;
	int i, r = -1;

	for (i = start_index; (i < nc) && (r < 0); i++) {
		if (ts.compare (i, ol, os) == 0) r = i;
	}
	return r;
}

void xds_append_string (void *o, void* other) {

	*((XalanDOMString*) o) += *((const XalanDOMString*) other);
}

void xds_put (void *o, int utf16, int index) {
	// set utf16 character
	// is ignoring const safe?

	XMLCh* src = const_cast<XMLCh*> (((XalanDOMString*) o)->c_str ());

	src[index] = utf16;
}

void xds_reverse (void *o) {

	XALAN_USING_XALAN(XalanDOMChar)

	XalanDOMString& s = *((XalanDOMString*) o);
	XalanDOMChar ch;

	XalanDOMString::iterator i = s.begin ();
	XalanDOMString::iterator e = s.end ();
	XalanDOMString::iterator ri = e - 1;
	int count;
	int nswap = s.length () / 2;

	for (count = 0; count < nswap; count++) {
		ch = *i;
		*i = *ri;
		*ri = ch;
		i++;
		ri--;
	}
}

// XalanDocumentPrefixResolver

void *xdpr_new_pr (void* doc) {

	XALAN_USING_XALAN(XalanDocument)
	XALAN_USING_XALAN(XalanDocumentPrefixResolver)

	XalanDocumentPrefixResolver* dpr = new XalanDocumentPrefixResolver ((XalanDocument*) doc);

	return dpr;
}

void xdpr_delete_pr (void* pr) {

	XALAN_USING_XALAN(XalanDocumentPrefixResolver)

	delete (XalanDocumentPrefixResolver*) pr;
}

// XPathEvaluator

void *xpe_new_pe () {

	XALAN_USING_XALAN(XPathEvaluator)

	XPathEvaluator *xpe = new XPathEvaluator;

	return xpe;
}

void xpe_delete_pe (void* pe) {

	XALAN_USING_XALAN(XPathEvaluator)

	delete (XPathEvaluator*) pe;
}

void *xpe_set_context (void *doc, void *pr, void *pe, void *sc) {

	XALAN_USING_XALAN(XalanDOMString)
	XALAN_USING_XALAN(XalanNode)
	XALAN_USING_XALAN(XalanDocument)
	XALAN_USING_XALAN(XalanDocumentPrefixResolver)
	XALAN_USING_XALAN(XPathEvaluator)

	XalanNode* myContextNode;

	myContextNode = ((XPathEvaluator *)pe)->selectSingleNode (
		*xstds,
		(XalanDocument*) doc,
		XalanDOMString ((char*)sc).c_str (),
		*((XalanDocumentPrefixResolver*) pr)
	);
	return myContextNode;
}

// Xalan Support

char get_whitespace () {

	return xstpl->getIncludeIgnorableWhitespace ();
}

void set_whitespace (char include) {

	xstpl->setIncludeIgnorableWhitespace (include);
}

void xml_parser_init () {

	// Initialize the XalanSourceTree subsystem...
	XALAN_USING_XALAN(XalanSourceTreeInit)
	XalanSourceTreeInit		theSourceTreeInit;

	// Initialize the XMLSupport subsystem...
 	XALAN_USING_XALAN(XMLSupportInit)
	XMLSupportInit			theXMLSupportInit;

	xstds = new XalanSourceTreeDOMSupport;

	XALAN_USING_XALAN(XalanSourceTreeParserLiaison)
	xstpl = new XalanSourceTreeParserLiaison (*xstds);

	// Hook the two together...

	xstds->setParserLiaison (xstpl);

//	cout << "Creating source tree support and parser liason support" << endl;
}

void xml_wrapper_finish () {

	if (xpl) delete xpl;
	if (xds) delete xds;
	if (efn) delete efn;
}

void xml_parser_finish () {

//	cout << "Deleting parser liason" << endl;
	delete xstpl;
//	cout << "Deleting source tree support" << endl;
	delete xstds;
}

void initialize_interface (char *level) {

	XALAN_USING_XERCES(XMLPlatformUtils)
	XALAN_USING_XERCES(DOMImplementationRegistry)
	XALAN_USING_XALAN(XalanTransformer)
	XALAN_USING_XALAN(XPathEvaluator)

	XMLPlatformUtils::Initialize(); // Xerces initialization
	XalanTransformer::initialize();
	XPathEvaluator::initialize();

	impl =  DOMImplementationRegistry::getDOMImplementation (TU (level));

//	cout << "Platform initialized: implementation level = " << level << endl;
}

void terminate_interface () {

	XALAN_USING_XALAN(XPathEvaluator)
	XALAN_USING_XERCES(XMLPlatformUtils)
	XALAN_USING_XALAN(XalanTransformer)

	XPathEvaluator::terminate();
	XalanTransformer::terminate();
	XalanTransformer::ICUCleanUp();
	XMLPlatformUtils::Terminate();

//	cout << "Platform terminated" << endl;
}

}

