#ifndef XALAN_SUPPORT_H
#define XALAN_SUPPORT_H

// Code is compiled as C++ but "C" headers are exported

// XalanNode

int xn_get_node_name (void *o);
int xn_get_node_value (void *o);
void *xn_get_node_value_utf (void *o);
int xn_get_node_type (void *o);
void *xn_get_owner_document (void *o); // returns *XalanDocument
void *xn_get_parent_node (void *o);
void *xn_get_child_nodes (void *o);
void *xn_get_first_child (void *o);
void *xn_get_last_child (void *o);
void *xn_get_previous_sibling (void *o);
void *xn_get_next_sibling (void *o);
void *xn_get_attributes (void *o);
void *xn_map_node (void *o);

// XalanElement

int xe_get_tag_name (void *o);

// XalanTransformer

void initialize ();
void terminate ();
void *new_transformer ();
void delete_transformer (void *t);
void parameter (void *t, char *key, char *expression);
void xt_set_indent (void *t, int indent);
void validation (void *t, int flag);
void *compile_stylesheet (void *t, char *fname);
void *parse_file (void *t, char *fname);
void *parse_string (void *t, char *buffer, int size);
void *dom_wrapper (void *t, void *dom);
void *string_stream ();
void *string_target (void *stream);
void *file_target (char *fname);
void delete_stringstream (void *stream);
void delete_target (void *target);
int feature_test (void *t, char *xml, char *xsl);
int transform_xml (int mode, void *t, void *xml, void *xsl, void *res);
int stream_length (void *stream);
void copy_stream (void *stream, char *strdata, int length);

// XalanDocument

void *parse_xml_file (char *filename);
void *parse_xml_mem (const char *buffer, int buffer_size);

void *new_parser_liaison ();
void delete_parser_liaison (void *pl);
void *wrap_xerces_doc (void *xerces_dom, void *pl);

void *xalan_root_element (void* doc);
void delete_xml_doc (void *doc);

// XPath

int xp_eval (void *pr, void *pe, void *cn, char *expr);
void *xp_eval_utf16 (void *pr, void *pe, void *cn, char *expr);

// NodeRefList

void *nrl_get_list (void *pe, void* cn, void *expr, void *doc);
int nrl_get_length (void *o);
void *nrl_item (void *o, int index);
void nrl_delete (void *nrl);

// XalanNodeList

int xnl_get_length (void *o);
void *xnl_item (void *o, int index);

//  XalanNamedNodeMap

int xnnm_get_length (void *o);
void *xnnm_item (void *o, int index);
void *xnnm_get_named_item (void *o, char *name);

//  XalanDOMString

void *new_xalan_dom_string (int npos);
void *make_xds_from_string (void *src);
void *copy_xalan_dom_string (void *src);
void delete_xalan_dom_string (void *object);

int xds_length (void *o);
void *xds_substr (void *o, void *dst, int posn, int n);
int xds_transcode (void *o);
int xds_at (void *o, int index); // return utf16 character
int xds_compare (void *o, void *other);
int xds_substring_index (void *o, void* other, int start_index);
void xds_append_string (void *o, void* other);
void xds_put (void *o, int utf16, int index); // set utf16 character
void xds_reverse (void *o);

// XalanDocumentPrefixResolver

void *xdpr_new_pr (void* doc);
void xdpr_delete_pr (void* pr);

// XPathEvaluator

void *xpe_new_pe ();
void xpe_delete_pe (void* pe);
void *xpe_set_context (void *doc, void *pr, void *pe, void *sc);

// Xalan Support

char get_whitespace ();
void set_whitespace (char include);

void xml_parser_init ();
void xml_wrapper_finish ();
void xml_parser_finish ();

// should be split...

void initialize_interface (char *level);
void terminate_interface ();

#endif
