indexing
	description:
		"Singleton which is instantiatied (and disposed) by the XML class"
		"A single instance of XERCES_DOM_PARSER is created as required."

class
	XML_SUPPORT

inherit
	CL_MISC
	UTF_SUPPORT

create
	make

feature {XML} -- Creation

	make is

		do
			initialize_interface (terminated ("Range"))
			xml_parser_init
			create node_factory.make
		end

	dispose is
 			-- C++ ftx object must be deleted before Xalan interface is terminated
		do
			if xdp /= Void then
				eh_delete (error_handler)
				xdp.dispose
			end
			if ftx /= Void then
				ftx.dispose
			end
			xml_wrapper_finish
			xml_parser_finish
			terminate_interface
		end

feature -- Access

	initialize is
			-- make is only called once
		do
		end

	include_ignorable_whitespace: BOOLEAN is
			--
		do
			Result := get_whitespace
		end

	set_include_ignorable_whitespace (include: BOOLEAN) is
			-- For a validating parser...
		do
			set_whitespace (include)
		end

feature -- Node factory

	node_factory: XALAN_NODE_FACTORY

	set_node_factory (factory: XALAN_NODE_FACTORY) is
			--
		require
			factory /= Void
		do
			node_factory := factory
		end

	create_dom_node (object: POINTER): DOM_NODE is
			--
		do
			Result := node_factory.create_dom_node (object)
		end

	create_xalan_node (object: POINTER): XALAN_NODE is
			--
		do
			Result := node_factory.create_xalan_node (object)
		end

feature -- DOM Parser

	xerces_dom_parser: XERCES_DOM_PARSER is
			--
		do
			if xdp = Void then
				create xdp.make
				error_handler := eh_create

				xdp.set_validation_scheme (xdp.Val_always)
				xdp.set_do_schema (true)
				xdp.set_do_namespaces (true)
				xdp.set_include_ignorable_whitespace (false)
				xdp.set_error_handler (error_handler)
			end
			Result := xdp
		end

	dom_parse_file (file_name: STRING): DOM_DOCUMENT is

		do
			Result := xerces_dom_parser.parse_file (file_name)
		end

	dom_parse_memory (xml_data: STRING): DOM_DOCUMENT is

		do
			Result := xerces_dom_parser.parse_memory (xml_data)
		end

feature -- Serialize

	ftx: FORMATTER_TO_XML
	ftx_output: STRING

	use_formatter_with_file (file_name: STRING) is

		require
			ftx_is (false)
		do
			ofstream := ofstream_create (terminated (file_name))
			create ftx.make (ofstream)
			set_formatter_defaults
		ensure
			ftx_is (true)
		end

	use_formatter_with_string is

		require
			ftx_is (false)
		do
			stringstream := stringstream_create
			create ftx.make (stringstream)
			set_formatter_defaults
		ensure
			ftx_is (true)
		end

	set_formatter_defaults is
			--
		require
			ftx_is (true)
		do
			ftx.set_do_indent (true)
			ftx.set_indent (2)
		end

	dispose_formatter is
			--
		require
			ftx_is (true)
		do
			ftx.dispose
			ftx := Void
			if ofstream /= default_pointer then
				ofstream_delete (ofstream)
				ofstream := default_pointer
			end
			if stringstream /= default_pointer then
				create ftx_output.make_filled (' ', stringstream_output_count (stringstream))
				stringstream_move_output (stringstream, str_to_ptr (ftx_output))
				stringstream_delete (stringstream)
				stringstream := default_pointer
			end
		ensure
			ftx_is (false)
		end

	ftx_is (flag: BOOLEAN): BOOLEAN is
			--
		local
			fs, ss: BOOLEAN
		do
			fs := ofstream /= default_pointer
			ss := stringstream /= default_pointer
			if flag then
				Result := (ftx /= Void) and (fs xor ss) -- only one stream
			else
				Result := (ftx = Void) and not (fs or ss) -- no streams
			end
		end

feature {} -- Private

	ofstream, stringstream: POINTER -- formatter output streams
	xdp: XERCES_DOM_PARSER
	error_handler: POINTER

feature {} -- Externals

	initialize_interface (level: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	terminate_interface is

		external
			"C | %"xalan_support.h%""
		end

	get_whitespace: BOOLEAN is

		external
			"C | %"xalan_support.h%""
		end

	set_whitespace (include: BOOLEAN) is

		external
			"C | %"xalan_support.h%""
		end

	xml_parser_init is

		external
			"C | %"xalan_support.h%""
		end

	xml_wrapper_finish is

		external
			"C | %"xalan_support.h%""
		end

	xml_parser_finish is

		external
			"C | %"xalan_support.h%""
		end

feature {} -- Error Handler

	eh_create: POINTER is

		external
			"C | %"dom_support.h%""
		end

	eh_delete (object: POINTER) is

		external
			"C | %"dom_support.h%""
		end

feature {} -- std C++

	ofstream_create (file_name: POINTER): POINTER is

		external
			"C | %"dom_support.h%""
		end

	ofstream_delete (object: POINTER) is

		external
			"C | %"dom_support.h%""
		end

	stringstream_create: POINTER is

		external
			"C | %"dom_support.h%""
		end

	stringstream_delete (object: POINTER) is

		external
			"C | %"dom_support.h%""
		end

	stringstream_output_count (object: POINTER): INTEGER is

		external
			"C | %"dom_support.h%""
		end

	stringstream_move_output (object, buffer: POINTER) is

		external
			"C | %"dom_support.h%""
		end

invariant
	paired_dom_parser: (xdp = Void) = (error_handler = default_pointer)
end
