indexing
	description:
		"XPath interface"

class
	XPATH

inherit
	XML export {} all end
	CL_MISC export {} all end
	UTF_SUPPORT
	DISPOSABLE

create
	xpath_make

feature {XALAN_DOCUMENT} -- Creation

	xpath_make (owner_document: XALAN_DOCUMENT) is

		do
			xdoc := owner_document
			dpr := xdpr_new_pr (xdoc.obj)
			xpe := xpe_new_pe
		end

	dispose is

		do
			if xpe /= default_pointer then
				xpe_delete_pe (xpe)
				xpe := default_pointer
			end
			if dpr /= default_pointer then
				xdpr_delete_pr (dpr)
				dpr := default_pointer
			end
			if xdoc /= Void then
				xdoc.remove_reference (Current)
				xdoc := Void
			end
		ensure then
			finished: xdoc = Void and xpe = default_pointer and dpr = default_pointer
		end

feature -- Access

	set_context_node (node: XALAN_NODE) is

		do
			cn := node.obj
		end

	set_context (expr: STRING) is
			-- Set context node from expression
		do
			cn := xpe_set_context (xdoc.obj, dpr, xpe, terminated (expr))
		end

	get_context_node: XALAN_NODE is
			--
		do
			create Result.make (cn)
		end

	good: BOOLEAN is
			--
		do
			Result := cn /= default_pointer
		end

	select_nodes (expression: STRING): NODE_REF_LIST is
			-- Needs a XpathEvaluator to work
		require
			has_context_node
		local
			p: POINTER
		do
			p := nrl_get_list (xpe, cn, terminated (expression), xdoc.obj)
			if p /= default_pointer then
				create Result.make (p)
			end
		end

	eval (expression: STRING): STRING is
			-- First function can have the side-effect
			-- of setting the transcode buffer
		require
			has_context_node
		do
			Result := get_transcoded (xp_eval (dpr, xpe, cn, terminated (expression)))
		end

	eval_utf (expression: STRING): XALAN_DOM_STRING is
			--
		require
			has_context_node
		do
			create Result.make (xp_eval_utf16 (dpr, xpe, cn, terminated (expression)))
		end

	count (expression: STRING): INTEGER is

		do
			Result := eval ("count(" + expression + ")").to_integer
		end

	has_context_node: BOOLEAN is
			--
		do
			Result := cn /= default_pointer
		end

feature {} -- Private

	xdoc: XALAN_DOCUMENT
	cn: POINTER -- Context node
	dpr, xpe: POINTER -- XalanDocumentPrefixResolver & XPathEvaluator

feature {} -- Expression evalution

	xp_eval (pr, pe, context_node, expr : POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	xp_eval_utf16 (pr, pe, context_node, expr : POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

feature {} -- XPathEvaluator

	xpe_new_pe: POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xpe_delete_pe (pe: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	xpe_set_context (document, prefix_resolver, path_evaluator, expression: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

feature {} -- XalanDocumentPrefixResolver

	xdpr_new_pr (document: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	xdpr_delete_pr (prefix_resolver: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

feature {} -- NodeRefList

	nrl_get_list (pe, context_node, expr, document: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

end
